#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import unittest

import RTC
import OpenRTM_aist
import DroneController
#import drone_distance_cal

import sys


from mock import Mock
from mock import patch, MagicMock

class TestDroneController(unittest.TestCase):
    def setUp(self):
        self.m = DroneController.DroneController(0)

        self.xx_logger = Mock()
        DroneController.logger.warning = self.xx_logger.warning
        DroneController.logger.error = self.xx_logger.error
        DroneController.logger.info = self.xx_logger.info
        DroneController.logger.debug = self.xx_logger.debug

    def test_read_mag(self):
        xx = Mock()
        self.m._inMagIn.isNew = xx.isNew
        self.m._inMagIn.read = xx.read

        init_mag = RTC.TimedFloat(RTC.Time(0, 0), 0.0)
        mag = RTC.TimedFloat(RTC.Time(0, 0), 45.0)

        self.m._d_inMag = init_mag
        xx.read.return_value = mag

        in_is_new = [False,True]
        result = [0, 45.0]

        for i,d in enumerate(in_is_new):
            xx.isNew.return_value = in_is_new[i]
            self.m._read_mag()
            
            self.assertEqual(self.m._d_inMag.data, result[i])

    def test_set_target_gps(self):
        xx = Mock()

        self.m._move_mode = xx._move_mode
        self.m._takeoff_landing = xx._takeoff_landing
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down
        self.m._move_continue = xx._move_continue
        #self.m._turn_dir = xx._turn_dir




        in_takeoff_landing = [1, -1, 0, 0, 0, 0, 0, 0, 0, 0,
                              0,  0, 0, 0, 0]

        in_home_lat = [37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 37.500, 
                       37.500, 37.500, 37.500, 37.500, 37.500]
        in_home_lon = [139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900, 139.900,
                       139.900, 139.900, 139.900, 139.900, 139.900]
        in_home_alt = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                       0.0, 0.0, 0.0, 0.0, 0.0]

        in_front_back = [0, 0, 0, 1, 1, -1, -1, 0, 0, 0,
                         0, 1, 1, -1, -1]
        in_right_left = [0, 0, 0, 0, 0, -1, -1, -1, -1, 1,
                        1, 0, 0, -1, -1]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 1, 1, 0,
                      0, 0, 0, 0, 0]

        in_move_continue = [False, False, False, False, False, False, False, False, False, True, True, True, True, True, True]

        in_drone_lat = [37.500, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520,
                        37.520, 37.520, 37.520, 37.520, 37.520]
        in_drone_lon = [139.900, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930,
                        139.930, 139.930, 139.930, 139.930, 139.930]
        in_drone_alt = [0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                        8.0, 8.0, 8.0, 8.0, 8.0]

        in_target_lat = [37.500, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.520, 37.530,
                        37.530, 37.530, 37.530, 37.5201, 37.5201]
        in_target_lon = [139.900, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.930, 139.940,
                        139.940, 139.940, 139.940, 139.9301, 139.9301]
        in_target_alt = [0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0, 8.0,
                        8.0, 8.0, 8.0, 8.0, 8.0]

        in_move_mode = ["Absolute", "Absolute", "Absolute", "Absolute", "Relative",
                        "Absolute", "Relative", "Absolute", "Relative", "Absolute",
                        "Relative", "Absolute", "Relative", "Absolute", "Relative"]
        in_mode = [1, 1, 1, 1, 2,
                   1, 2, 1, 2, 1,
                   2, 1, 2, 1, 2]

        in_last_speed = [3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0, 3.0,
                         3.0, 4.0, 4.0, 3.0, 3.0]

        
        in_last_front_back = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                              0.0, 1.0, 1.0, -1.0, -1.0]
        in_last_right_left = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0,
                              1.0, 0.0, 0.0, -1.0, -1.0]
        in_last_up_down = [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0.0, 0.0, 0.0, 0.0]

        
        target_lat_result = [37.500, 37.500, 37.520, 37.52041, 37.52019, 37.51959, 37.520, 37.520, 37.52019, 37.530,
                           37.530, 37.52041, 37.52019, 37.51959, 37.520]

        target_lon_result = [139.900, 139.900, 139.930, 139.930, 139.93019, 139.92968, 139.92973, 139.92968, 139.92981, 139.940,
                           139.940, 139.930, 139.93019, 139.92968, 139.92973]

        target_alt_result = [10.0, 0.0, 8.0, 8.0, 8.0, 8.0, 8.0, 9.0, 9.0, 8.0,
                                8.0, 8.0, 8.0, 8.0, 8.0]

        self.m._home_gps = True
        self.m._drone_gps = True
        self.m._SpeedParam = 3.0
        self.m._d_inMag.data = 45
        test_case = 0
        while test_case < len(target_lat_result):
            #mode
            xx._move_mode.return_value = in_mode[test_case]
            self.m._set_move_mode()

            #gps
            xx._takeoff_landing.return_value = in_takeoff_landing[test_case]
            xx._front_back.return_value =in_front_back[test_case]
            xx._right_left.return_value =in_right_left[test_case]
            xx._up_down.return_value =in_up_down[test_case]
            xx._move_continue.return_value = in_move_continue[test_case]

            self.m._last_front_back = in_last_front_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]

            self.m._d_HomeGPSIn.latitude = in_home_lat[test_case]
            self.m._d_HomeGPSIn.longitude = in_home_lon[test_case]
            self.m._d_HomeGPSIn.altitude = in_home_alt[test_case]

            self.m._d_DroneGPSIn.latitude = in_drone_lat[test_case]
            self.m._d_DroneGPSIn.longitude = in_drone_lon[test_case]
            self.m._d_DroneGPSIn.altitude = in_drone_alt[test_case]

            self.m._d_TargetGPSOut.latitude = in_target_lat[test_case]
            self.m._d_TargetGPSOut.longitude = in_target_lon[test_case]
            self.m._d_TargetGPSOut.altitude = in_target_alt[test_case]

            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._last_speedparam = in_last_speed[test_case]

            self.m._set_target_gps()


            self.assertEqual(round(self.m._d_TargetGPSOut.latitude,5), target_lat_result[test_case])
            self.assertEqual(round(self.m._d_TargetGPSOut.longitude,5), target_lon_result[test_case])
            self.assertEqual(self.m._d_TargetGPSOut.altitude, target_alt_result[test_case])

            test_case += 1

    def test_set_target_speed(self):
        xx = Mock()

        init_speed = RTC.TimedFloat(RTC.Time(0, 0), 0.0)


        self.m._d_outTargetSpeed = init_speed


        speed = [5.0]
        result = [5.0]

        self.m._SpeedParam = speed[0]
        self.m._set_target_speed()
        self.assertEqual(self.m._d_outTargetSpeed.data, result[0])



    def test_set_target_yaw(self):
        xx = Mock()

        self.m._turn_dir = xx._turn_dir
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down
        self.m._move_continue = xx._move_continue
        self.m._diff_target_angle = xx._diff_target_angle

        init_yaw = RTC.TimedShortSeq(RTC.Time(0, 0), [])

        self.m._d_outTargetYaw = init_yaw

        in_magcheck = [False, True, True, True, True, True, True, True, True, True,
                       True, True, True, True, True]
        in_turn_dir = [0, 0, 0, -1, 1, -1, 1, 0, 0, 0,
                      0, 0, 0, 0, 0]
        in_front_back = [0, 0, 0, 0, 0, -1, 0, -1, 1, 0,
                         0, -1, 1, 0, 0]
        in_right_left = [0, 0, 0, 0, 0, 0, 1, 0, 0, -1,
                         1, 0, 0, -1, 1]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                      0, 0, 0, 0, 0]

        in_move_continue = [False, False, False, False, False, False, False, False, False, False,
                            False, False, False, True, True]
        in_relative_mag = [45, 45, 45, 45, 45, 45, 45, 45, 45, 45,
                           45, 45, 45, 45, 45]
        in_mag = [45, 45, 45, 45, 45, 45, 45, 45, 45, 45,
                  45, 50, 50, 50, 50]
        in_move_mode = ["Absolute", "Absolute", "Relative", "Absolute", "Relative",
                        "Relative", "Relative", "Absolute", "Absolute", "Absolute",
                        "Absolute", "Relative", "Relative", "Relative", "Relative"]
        in_diff_target_angle = [0,0,0,0,0,0,0, 135, -45, -135,
                                45,0,0,0,0]


        moveturnspeed = [30]
        turnspeed = [45]
        self.m._MoveTurnSpeed[0] = moveturnspeed[0]
        self.m._TurnSpeed[0] = turnspeed[0]

        result = [[], [0, 0], [0, 0], [-1, 0], [1,  90],
                  [-1, 15.0], [1, 75], [1, 180], [-1, 0], [-1, 270],
                  [ 1,   90], [ 1,   50], [ 1,  50], [ 1,  45], [ 1, 45]
                 ]

        test_case = 0
        while test_case < len(in_mag):
            xx._turn_dir.return_value = in_turn_dir[test_case]
            xx._front_back.return_value = in_front_back[test_case]
            xx._right_left.return_value = in_right_left[test_case]
            xx._up_down.return_value = in_up_down[test_case]
            xx._move_continue.return_value = in_move_continue[test_case]
            xx._diff_target_angle.return_value = in_diff_target_angle[test_case]

            self.m._mag = in_magcheck[test_case]
            self.m._relative_mag = in_relative_mag[test_case]
            self.m._d_inMag.data = in_mag[test_case]
            self.m._d_outMoveMode.data = in_move_mode[test_case]

            self.m._set_target_yaw()

            self.assertEqual(self.m._d_outTargetYaw.data, result[test_case])

            test_case += 1

    def test_set_move_mode(self):
        xx = Mock()

        self.m._move_mode = xx._move_mode

        init_movemode = RTC.TimedString(RTC.Time(0, 0), "")

        self.m._d_outMoveMode = init_movemode

        in_move = [1, 2]
        result = ["Absolute", "Relative"]

        for i, d in enumerate(in_move):
            xx._move_mode.return_value = in_move[i]
            self.m._set_move_mode()
            self.assertEqual(self.m._d_outMoveMode.data, result[i])

    def test_diff_target_angle(self):
        in_mag = [30, 30, 280, 280]
        in_target_lat = [37.520, 37.580, 37.520, 37.580]
        in_target_lon = [139.910, 139.810, 139.910, 139.810]
        in_pos_lat = [37.550, 37.500, 37.550, 37.500]
        in_pos_lon = [139.940, 139.890, 139.940, 139.890]

        deg_result = [-165.0, -75.0, -55.0, 35.0]

        test_case = 0
        while test_case < len(deg_result):

            self.m._d_inMag.data = in_mag[test_case]

            self.m._d_TargetGPSOut.latitude = in_target_lat[test_case]
            self.m._d_TargetGPSOut.longitude = in_target_lon[test_case]
            self.m._d_DroneGPSIn.latitude = in_pos_lat[test_case]
            self.m._d_DroneGPSIn.longitude = in_pos_lon[test_case]

            deg = round(self.m._diff_target_angle(), 1)

            self.assertEqual(deg, deg_result[test_case])

            test_case += 1

    def test_move_mode(self):
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home

        in_home = [True, True, True, True, False, False, False, False, False, False, False, False]
        in_target_mode = [1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2]
        in_button = [0, 0, DroneController.BUTTON_10_MASK, DroneController.BUTTON_10_MASK,
                     0, 0, DroneController.BUTTON_10_MASK, DroneController.BUTTON_10_MASK,
                     0, 0, DroneController.BUTTON_10_MASK, DroneController.BUTTON_10_MASK]

        result = [1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 1, 1]

        test_case = 0
        while test_case < len(result):
            xx._check_drone_on_home.return_value = in_home[test_case]
            self.m._target_gps._mode = in_target_mode[test_case]
            self.m._d_ButtonIn.data = in_button[test_case]

            mode = self.m._move_mode()

            self.assertEqual(mode, result[test_case])

            test_case += 1

    def test_front_back(self):
        in_data = [[], [-2,0], [-1,0], [0,0], [1,0], [2,0]]
        result =  [ 0,      1,      1,     0,    -1,   -1]

        for i, d in enumerate(in_data):
            self.m._d_AnalogIn.data = in_data[i]

            self.assertEqual(self.m._front_back(), result[i])

    def test_move_continue(self):
        xx = Mock()

        self.m._turn_dir = xx._turn_dir        

        in_last_move_mode = ["Absolute", "Relative", "Absolute", "Absolute", "Absolute",
                             "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute"]
        in_move_mode = ["Relative", "Absolute", "Absolute", "Absolute", "Absolute",
                        "Absolute", "Absolute", "Absolute", "Absolute", "Absolute", "Absolute"]

        in_turn_dir = [0, 0, -1, 1, 0, 0, 0, 0, 0, 0, 0]

        in_last_flont_back = [0, 1, 0, 0, 0, 0, -1, 0, 0, 1, -1]
        in_flont_back = [0, 1, 0, 0, 0, 0, 0, 1, 0, 1, -1]

        in_last_right_left = [0, 0, 0, -1, 0, 0, 0, 1, -1, 1, 1]
        in_right_left = [0, 0, 0, -1, 0, 0, 1, 1, -1, 1, 1]

        in_last_up_down = [0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 1]
        in_up_down = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]



        result = [False, False, False, False, False, False, False, False, True, True, True]


        test_case = 0
        while test_case < len(result):
            xx._turn_dir.return_value = in_turn_dir[test_case]

            self.m._last_move_mode = in_last_move_mode[test_case]
            self.m._d_outMoveMode.data = in_move_mode[test_case]
            self.m._last_front_back = in_last_flont_back[test_case]
            self.m._last_right_left = in_last_right_left[test_case]
            self.m._last_up_down = in_last_up_down[test_case]

            moved_countinue = self.m._move_continue(in_flont_back[test_case], in_right_left[test_case], in_up_down[test_case])

            self.assertEqual(moved_countinue, result[test_case])

            test_case += 1

    def test_turn_dir(self):  
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home
        self.m._front_back = xx._front_back
        self.m._right_left = xx._right_left
        self.m._up_down = xx._up_down


        in_home = [False, True]
        in_data = [[0,-1], [0,0], [0,1]]
        in_move = [-1, 0, 1]

        result =  [[[[[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, -1, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[ 0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0,  1, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]],


                   [[[[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]],
                   [[[ 0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]], [[0, 0, 0],[0, 0, 0],[0, 0, 0]]]]
                    ]

        self.m._d_outMoveMode.data = "Absolute"
        for i1, d1 in enumerate(in_home):
            xx._check_drone_on_home.return_value = in_home[i1]
            for i2, d2 in enumerate(in_data):
                self.m._d_AnalogIn.data = in_data[i2]
                for i3, d3 in enumerate(in_move):
                    #front_back
                    xx._front_back.return_value =in_move[i3]
                    for i4, d4 in enumerate(in_move):
                        #right_left
                        xx._right_left.return_value = in_move[i4]
                        for i5, d5 in enumerate(in_move):
                            #up_down
                            xx._up_down.return_value = in_move[i5]

                            operation = self.m._turn_dir()

                            self.assertEqual(operation, result[i1][i2][i3][i4][i5])

    def test_move_speed(self):
        xx = Mock()

        self.m._check_drone_on_home = xx._check_drone_on_home

        in_home = [False, True] 
        in_button = [0,
                     DroneController.BUTTON_05_MASK,
                     DroneController.BUTTON_07_MASK,
                     DroneController.BUTTON_05_MASK | DroneController.BUTTON_07_MASK
                     ]

        in_speed = [1.0, 2.0, 5.0,  9.0, 10.0]
        result1 = [[[1.0, 2.0, 5.0,  9.0, 10.0],
                   [2.0, 3.0, 6.0, 10.0, 10.0],
                   [1.0, 1.0, 4.0,  8.0,  9.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]],

                   [[1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0],
                   [1.0, 2.0, 5.0,  9.0, 10.0]]
                   ]
        result2 = [[[0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0],
                   [0, 0, 0, 0, 0],
                   [1, 1, 1, 1, 1]],

                   [[0, 0, 0, 0, 0],
                   [1, 1, 1, 1, 1],
                   [1, 1, 1, 1, 1],
                   [1, 1, 1, 1, 1]]
                   ]

        for i1, d1 in enumerate(in_home):
            xx._check_drone_on_home.return_value = in_home[i1]        
            for i2, d2 in enumerate(in_button):
                self.m._d_ButtonIn.data = in_button[i2]
                for i3, d3 in enumerate(in_speed):
                    self.xx_logger.warning.call_count = 0
                    self.m._SpeedParam = in_speed[i3]

                    self.m._move_speed()

                    self.assertEqual(self.m._SpeedParam, result1[i1][i2][i3])
                    self.assertEqual(self.xx_logger.warning.call_count, result2[i1][i2][i3])

    def test_front_back_check(self):
        in_status = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_status):
            status = self.m._front_back_check(in_status[i])
            self.assertEqual(status, result[i])

    def test_turn_dir_check(self):
        in_dir = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_dir):
            dir = self.m._turn_dir_check(in_dir[i])
            self.assertEqual(dir, result[i])
            
    def test_move_speed_check(self):
        in_speed = [0.0, 1.0, 2.0, 5.0, 9.0, 10.0, 11.0]
        result  =  [1.0, 1.0, 2.0, 5.0, 9.0, 10.0, 10.0]
        
        for i, d in enumerate(in_speed):
            speed = self.m._move_speed_check(in_speed[i])
            self.assertEqual(speed, result[i])
            
                        
if __name__ == '__main__':
    unittest.main()
