#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import unittest

import RTC
import target_gps

import sys

from mock import Mock
from mock import patch, MagicMock


class TestTargetGPS(unittest.TestCase):
    def setUp(self):
        self.m = target_gps.TargetGPS()
        
        xx = Mock()
        target_gps.logger.warning = xx.warning
    
    def test_config_param(self):
        config = {}
        
        self.m.config_param(config)
        self.assertEqual(self.m._AltCoefficient, 1)
        self.assertEqual(self.m._LatLonCoefficient, 2)
        self.assertEqual(self.m._LatDiff10m, 0.000137)
        self.assertEqual(self.m._LonDiff10m, 0.000108)
        self.assertEqual(self.m._MoveSpeed, 3.0)
        self.assertEqual(self.m._MinAlt, 2.0)
        self.assertEqual(self.m._MaxAlt, 15.0)
        self.assertEqual(self.m._TurnSpeed, 45)
        self.assertEqual(self.m._NoseLatDiff10m, 9.01323)
        self.assertEqual(self.m._NoseLonDiff10m, 8.9831566)
        
        key1 = 111
        key2 = 222
        key3 = 333
        key4 = 444
        key5 = 555
        key6 = 666
        key7 = 777
        key8 = 888
        key9 = 999
        key10 = 1010
        
        config = {"AltCoefficient": key1,
                  "LatLonCoefficient": key2,
                  "LatDiff10m": key3,
                  "LonDiff10m": key4,
                  "MoveSpeed": key5,
                  "MinAlt": key6,
                  "MaxAlt": key7,
                  "TurnSpeed": key8,
                  "NoseLatDiff10m": key9,
                  "NoseLonDiff10m": key10
                  }
        
        self.m.config_param(config)
        
        self.assertEqual(self.m._AltCoefficient, key1)
        self.assertEqual(self.m._LatLonCoefficient, key2)
        self.assertEqual(self.m._LatDiff10m, key3)
        self.assertEqual(self.m._LonDiff10m, key4)
        self.assertEqual(self.m._MoveSpeed, key5)
        self.assertEqual(self.m._MinAlt, key6)
        self.assertEqual(self.m._MaxAlt, key7)
        self.assertEqual(self.m._TurnSpeed, key8)
        self.assertEqual(self.m._NoseLatDiff10m, key9)
        self.assertEqual(self.m._NoseLonDiff10m, key10)

    def test_set_controller_mode(self):
        in_controller_mode = [1, 2]
        result = [1, 2]
        
        for i, d in enumerate(in_controller_mode):
            self.m.set_controller_mode(in_controller_mode[i])
            self.assertEqual(self.m._mode, result[i])
            
    def test_move_south_north(self):
        self.m._init_variable()
        
        in_speedparam = [2.0, 5.0]
        in_south_north = [-2,-1,0,1,2]
        in_drone_lat = [-89.999980, 37.055183, 37.055183, 37.055183, 89.999980]
        result_lat = [[-90.000000, 37.055128, 37.055183, 37.055238, 90.000000],
                      [-90.000000, 37.055046, 37.055183, 37.055320, 90.000000]
                      ]
        
        for i1, d2 in enumerate(in_speedparam):
            for i2, d2 in enumerate(in_south_north):
                moved_lat = self.m._move_south_north(in_drone_lat[i2], in_south_north[i2], in_speedparam[i1])
                self.assertEqual(round(moved_lat, 6), result_lat[i1][i2])
    
    def test_move_east_west(self):    
        self.m._init_variable()
        
        in_speedparam = [2.0, 5.0]
        in_east_west = [-2,-1,0,1,2]
        in_drone_lon = [-179.999970, 140.892362, 140.892362, 140.892362, 179.999960]
        result_lon = [[179.999987, 140.892319, 140.892362, 140.892405, -179.999997],
                      [179.999922, 140.892254, 140.892362, 140.892470, -179.999932]
                      ]
        
        for i1, d1 in enumerate(in_speedparam):
            for i2, d2 in enumerate(in_east_west):
                moved_lon = self.m._move_east_west(in_drone_lon[i2], in_east_west[i2], in_speedparam[i1])
                self.assertEqual(round(moved_lon, 6), result_lon[i1][i2])
           
    def test_move_nose(self):
        drone = RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                            0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                            0.0, 0, RTC.GPS_FIX_NORMAL)
        
        self.m._init_variable()
        
        in_flont_back = [-1, 0, 1]
        in_right_left = [-1, 0, 1]
        in_drone_lat = [-89.999999, 37.523763, 89.999999]
        in_drone_lon = [-179.999999, 139.938025, 179.999999]
        in_speedparam = [3.0]
        in_mag = [45.0]
        
        result = [
                  [
                   [[[-89.999999,179.999947,], [-89.999999,139.937971], [-89.999999,179.999945]],   [[37.523763,179.999947,],[37.523763,139.937971],[37.523763,179.999945]],   [[89.999999,179.999947,],[89.999999,139.937971],[89.999999,179.999945]]], 
                   [[[-90.000000,179.999963,],  [-90.000000,139.937987], [-90.000000,179.999961]],   [[37.523725,179.999963,],[37.523725,139.937987],[37.523725,179.999961]],   [[89.999961,179.999963,],[89.999961,139.937987],[89.999961,179.999961]]],
                   [[[-90.000000,-179.999999,], [-90.000000,139.938025], [-90.000000,179.999999]],   [[37.523709,-179.999999,],[37.523709,139.938025],[37.523709,179.999999]],   [[89.999945,-179.999999,],[89.999945,139.938025],[89.999945,179.999999]]]
                  ],
                  [
                   [[[-89.999961,179.999963,],  [-89.999961,139.937987], [-89.999961,179.999961]],   [[37.523801,179.999963,],[37.523801,139.937987],[37.523801,179.999961]],   [[90.000000,179.999963,],[90.000000,139.937987],[90.000000,179.999961]]], 
                   [[[-89.999999,-179.999999,], [-89.999999,139.938025], [-89.999999,179.999999]],   [[37.523763,-179.999999,],[37.523763,139.938025],[37.523763,179.999999]],   [[89.999999,-179.999999,],[89.999999,139.938025],[89.999999,179.999999]]],
                   [[[-90.000000,-179.999961,], [-90.000000,139.938063], [-90.000000,-179.999963]],   [[37.523725,-179.999961,],[37.523725,139.938063],[37.523725,-179.999963]],   [[89.999961,-179.999961,],[89.999961,139.938063],[89.999961,-179.999963]]]
                  ],
                  [
                   [[[-89.999945,-179.999999,], [-89.999945,139.938025], [-89.999945,179.999999]],   [[37.523817,-179.999999,],[37.523817,139.938025],[37.523817,179.999999]],   [[90.000000,-179.999999,],[90.000000,139.938025],[90.000000,179.999999]]], 
                   [[[-89.999961,-179.999961,], [-89.999961,139.938063], [-89.999961,-179.999963]],   [[37.523801,-179.999961,],[37.523801,139.938063],[37.523801,-179.999963]],   [[90.000000,-179.999961,],[90.000000,139.938063],[90.000000,-179.999963]]],
                   [[[-89.999999,-179.999945,], [-89.999999,139.938079], [-89.999999,-179.999947]],   [[37.523763,-179.999945,],[37.523763,139.938079],[37.523763,-179.999947]],   [[89.999999,-179.999945,],[89.999999,139.938079],[89.999999,-179.999947]]]
                  ]
                 ]
        
        for i1, d1 in enumerate(in_flont_back):
            for i2, d2 in enumerate(in_right_left):
                for i3, d3 in enumerate(in_drone_lat):
                    drone.latitude = in_drone_lat[i3]
                    for i4, d4 in enumerate(in_drone_lon):
                        drone.longitude = in_drone_lon[i4]
                        moved_latlon = self.m._move_nose(drone, in_flont_back[i1], in_right_left[i2], in_speedparam[0], in_mag[0])
                        moved_latlon[0] = round(moved_latlon[0], 6)
                        moved_latlon[1] = round(moved_latlon[1], 6)

                        self.assertEqual(moved_latlon, result[i1][i2][i3][i4])


    def test_button_status_check(self):
        in_button_status = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_button_status):
            button = self.m._button_status_check(in_button_status[i])
            self.assertEqual(button, result[i])
            
    def test_analog_status_check(self):
        in_analog_status = [-2, -1, 0, 1, 2]
        result = [-1, -1, 0, 1, 1]
        
        for i, d in enumerate(in_analog_status):
            analog = self.m._analog_status_check(in_analog_status[i])
            self.assertEqual(analog, result[i])

    def test_alt_range_check(self):
        in_alt = [0.0, 1.0, 2.0, 3.0, 9.0, 14.0, 15.0, 16.0, 17.0]
        result = [2.0, 2.0, 2.0, 3.0, 9.0, 14.0, 15.0, 15.0, 15.0]
        
        for i, d in enumerate(in_alt):
            alt = self.m._alt_range_check(in_alt[i])
            self.assertEqual(alt, result[i])

    def test_lat_range_check(self):
        in_lat = [-91.000000, -90.000001, -90.000000, -89.999999, -80.000000, 0.0, 80.000000, 89.999999, 90.000000, 90.000001, 91.000000]
        result = [-90.000000, -90.000000, -90.000000, -89.999999, -80.000000, 0.0, 80.000000, 89.999999, 90.000000, 90.000000, 90.000000]
        
        for i, d in enumerate(in_lat):
            lat = self.m._lat_range_check(in_lat[i])
            self.assertEqual(lat, result[i])
            
    def test_lon_range_check(self):
        in_lon = [-181.000000, -180.000001, -180.000000, -179.999999, -170.000000, -90.000000, 0.0, 90.000000, 170.000000, 179.999999, 180.000000,  180.000001,  181.000000]
        result = [ 179.000000,  179.999999, -180.000000, -179.999999, -170.000000, -90.000000, 0.0, 90.000000, 170.000000, 179.999999, 180.000000, -179.999999, -179.000000]
        
        for i, d in enumerate(in_lon):
            lon = self.m._lon_range_check(in_lon[i])
            self.assertEqual(lon, result[i])

if __name__ == '__main__':
    unittest.main()