#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file DroneMapViewer.py
 @brief Display the position of the drone.
 @date $Date$

 @author East Japan Accounting Center Co., Ltd

 BSD 3clause "New" or "Revised" License

"""
import sys

# Import RTM module
import RTC
import OpenRTM_aist

# Import map_create
import map_create

# Setup logger
from logging import getLogger, StreamHandler
from logging import DEBUG as LOG_LV

sys.path.append(".")

logger = getLogger(__name__)
handler = StreamHandler()
logger.setLevel(LOG_LV)
handler.setLevel(LOG_LV)
logger.addHandler(handler)

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
dronemapviewer_spec = ["implementation_id", "DroneMapViewer", 
         "type_name",         "DroneMapViewer", 
         "description",       "Display the position of the drone.", 
         "version",           "1.0.0", 
         "vendor",            "EAC", 
         "category",          "Drone", 
         "activity_type",     "STATIC", 
         "max_instance",      "1", 
         "language",          "Python", 
         "lang_type",         "SCRIPT",
         "conf.default.MapZoomLevel", "18",
         "conf.default.DroneNum", "3",
         "conf.__widget__.MapZoomLevel", "spin",
         "conf.__widget__.DroneNum", "spin",
         "conf.__constraints__.MapZoomLevel", "0<=x<=19",
         "conf.__constraints__.DroneNum", "1<=x<=3",
         ""]
# </rtc-template>

##
# @class DroneMapViewer
# @brief Display the position of the drone.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# Refer to the "機能仕様書_DroneMapViewer" for details.
# 
# 
class DroneMapViewer(OpenRTM_aist.DataFlowComponentBase):
    
    ##
    # @brief constructor
    # @param manager Maneger Object
    # 
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_Drone1GPSin = RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                          0.0, 0, RTC.GPS_FIX_NORMAL)
        """
        ドローン現在値座標　　　（１号機）
         - Type: RTC::GPSData
        """
        self._Drone1GPSinIn = OpenRTM_aist.InPort("Drone1GPSin", self._d_Drone1GPSin)
        self._d_Target1GPSin = RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                          0.0, 0, RTC.GPS_FIX_NORMAL)
        """
        ターゲット座標　　　（１号機）
         - Type: RTC::GPSData
        """
        self._Target1GPSinIn = OpenRTM_aist.InPort("Target1GPSin", self._d_Target1GPSin)
        self._d_inMag1 = RTC.TimedFloat(RTC.Time(0,0),0)
        """
        磁気コンパス　　　（１号機）
         - Type: RTC::TimedFloat
        """
        self._inMag1In = OpenRTM_aist.InPort("inMag1", self._d_inMag1)
        self._d_inState1 = RTC.TimedString(RTC.Time(0,0),0)
        """
        ドローンI/Oの内部ステータス　　　（１号機）
         - Type: RTC::TimedString
        """
        self._inState1In = OpenRTM_aist.InPort("inState1", self._d_inState1)
        self._d_Drone2GPSin = RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                          0.0, 0, RTC.GPS_FIX_NORMAL)
        """
        ドローン現在値座標　　　（２号機）
         - Type: RTC::GPSData
        """
        self._Drone2GPSinIn = OpenRTM_aist.InPort("Drone2GPSin", self._d_Drone2GPSin)
        self._d_Target2GPSin = RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                          0.0, 0, RTC.GPS_FIX_NORMAL)
        """
        ターゲット座標　　　（２号機）
         - Type: RTC::GPSData
        """
        self._Target2GPSinIn = OpenRTM_aist.InPort("Target2GPSin", self._d_Target2GPSin)
        self._d_inMag2 = RTC.TimedFloat(RTC.Time(0,0),0)
        """
        磁気コンパス　　　（２号機）
         - Type: RTC::TimedFloat
        """
        self._inMag2In = OpenRTM_aist.InPort("inMag2", self._d_inMag2)
        self._d_inState2 = RTC.TimedString(RTC.Time(0,0),0)
        """
        ドローンI/Oの内部ステータス　　　（２号機）
         - Type: RTC::TimedString
        """
        self._inState2In = OpenRTM_aist.InPort("inState2", self._d_inState2)
        self._d_Drone3GPSin = RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                          0.0, 0, RTC.GPS_FIX_NORMAL)
        """
        ドローン現在値座標　　　（３号機）
         - Type: RTC::GPSData
        """
        self._Drone3GPSinIn = OpenRTM_aist.InPort("Drone3GPSin", self._d_Drone3GPSin)
        self._d_Target3GPSin = RTC.GPSData(RTC.Time(0,0), RTC.GPSTime(0,0),
                                          0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                                          0.0, 0, RTC.GPS_FIX_NORMAL)
        """
        ターゲット座標　　　（３号機）
         - Type: RTC::GPSData
        """
        self._Target3GPSinIn = OpenRTM_aist.InPort("Target3GPSin", self._d_Target3GPSin)
        self._d_inMag3 = RTC.TimedFloat(RTC.Time(0,0),0)
        """
        磁気コンパス　　　（３号機）
         - Type: RTC::TimedFloat
        """
        self._inMag3In = OpenRTM_aist.InPort("inMag3", self._d_inMag3)
        self._d_inState3 = RTC.TimedString(RTC.Time(0,0),0)
        """
        ドローンI/Oの内部ステータス　　　（３号機）
         - Type: RTC::TimedString
        """
        self._inState3In = OpenRTM_aist.InPort("inState3", self._d_inState3)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">
        """
        地図の縮尺レベル
         - Name: MapZoomLevel MapZoomLevel
         - DefaultValue: 18
         - Range: 0<=x<=19
        """
        self._MapZoomLevel = [18]
        """
        表示するドローンの数
         - Name: DroneNum DroneNum
         - DefaultValue: 3
         - Range: 1<=x<=3
        """
        self._DroneNum = [3]
        
        # </rtc-template>

        inport_num = 3
        self._d_DroneGPSin = [0] * inport_num
        self._DroneGPSinIn = [0] * inport_num
        self._d_TargetGPSin = [0] * inport_num
        self._TargetGPSinIn = [0] * inport_num
        self._d_inMag = [0] * inport_num
        self._inMagIn = [0] * inport_num
        self._d_inState = [0] * inport_num
        self._inStateIn = [0] * inport_num

        self._d_DroneGPSin[0] = self._d_Drone1GPSin
        self._DroneGPSinIn[0] = self._Drone1GPSinIn
        self._d_DroneGPSin[1] = self._d_Drone2GPSin
        self._DroneGPSinIn[1] = self._Drone2GPSinIn
        self._d_DroneGPSin[2] = self._d_Drone3GPSin
        self._DroneGPSinIn[2] = self._Drone3GPSinIn

        self._d_TargetGPSin[0] = self._d_Target1GPSin
        self._TargetGPSinIn[0] = self._Target1GPSinIn
        self._d_TargetGPSin[1] = self._d_Target2GPSin
        self._TargetGPSinIn[1] = self._Target2GPSinIn
        self._d_TargetGPSin[2] = self._d_Target3GPSin
        self._TargetGPSinIn[2] = self._Target3GPSinIn

        self._d_inMag[0] = self._d_inMag1
        self._inMagIn[0] = self._inMag1In
        self._d_inMag[1] = self._d_inMag2
        self._inMagIn[1] = self._inMag2In
        self._d_inMag[2] = self._d_inMag3
        self._inMagIn[2] = self._inMag3In

        self._d_inState[0] = self._d_inState1
        self._inStateIn[0] = self._inState1In
        self._d_inState[1] = self._d_inState2
        self._inStateIn[1] = self._inState2In
        self._d_inState[2] = self._d_inState3
        self._inStateIn[2] = self._inState3In

        self._drone_gps = [False] * self._DroneNum[0]
        self._target_gps = [False] * self._DroneNum[0]
        self._mag = [False] * self._DroneNum[0]
        self._state = [False] * self._DroneNum[0]

        self.drone_num_set = map_create.DroneNumSet()
        self.drone_pos_update = map_create.DronePosUpdate()
        self.drone_angle_update = map_create.DroneAngleUpdate()
        self.target_pos_update = map_create.TargetPosUpdate()
        self.drone_state_update = map_create.DroneStateUpdate()
        self.map_zoom_update = map_create.MapZoomUpdate()

    ##
    # RTC初期化
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry() 
    # 
    # @return RTC::ReturnCode_t
    # 
    #
    def onInitialize(self):
        # Bind variables and configuration variable
        self.bindParameter("MapZoomLevel", self._MapZoomLevel, "18")
        self.bindParameter("DroneNum", self._DroneNum, "3")
        
        # Set InPort buffers
        self.addInPort("Drone1GPSin",self._Drone1GPSinIn)
        self.addInPort("Target1GPSin",self._Target1GPSinIn)
        self.addInPort("inMag1",self._inMag1In)
        self.addInPort("inState1",self._inState1In)
        self.addInPort("Drone2GPSin",self._Drone2GPSinIn)
        self.addInPort("Target2GPSin",self._Target2GPSinIn)
        self.addInPort("inMag2",self._inMag2In)
        self.addInPort("inState2",self._inState2In)
        self.addInPort("Drone3GPSin",self._Drone3GPSinIn)
        self.addInPort("Target3GPSin",self._Target3GPSinIn)
        self.addInPort("inMag3",self._inMag3In)
        self.addInPort("inState3",self._inState3In)
        
        # Set OutPort buffers
        
        # Set service provider to Ports
        
        # Set service consumers to Ports
        
        # Set CORBA Service Ports
        
        return RTC.RTC_OK
    
    #    ##
    #    # 
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    # 
    #    # @return RTC::ReturnCode_t
    #
    #    # 
    #def onFinalize(self):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    # 
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK
    
        ##
        # ドローン台数の決定
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        # 
        # @return RTC::ReturnCode_t
        #
        #
    def onActivated(self, ec_id):
        self._Drone_Num = self._DroneNum[0]
        self.drone_num_set.drone_num(self._Drone_Num)

        return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The deactivated action (Active state exit action)
    #    # former rtc_active_exit()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onDeactivated(self, ec_id):
    #
    #    return RTC.RTC_OK
    
        ##
        # ドローンGPS、MAGデータ取得およびマップビューアの表示
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        #
    def onExecute(self, ec_id):
        self._read_inport()

        self._set_map_data()

        return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The aborting action when main logic error occurred.
    #    # former rtc_aborting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onAborting(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onError(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The reset action that is invoked resetting
    #    # This is same but different the former rtc_init_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onReset(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    #def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK
    
    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    #def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    def _read_inport(self):
        self._drone_gps = [False] * self._Drone_Num
        self._target_gps = [False] * self._Drone_Num
        self._mag = [False] * self._Drone_Num
        self._state = [False] * self._Drone_Num

        num = 0
        while num < self._Drone_Num:
            # read_drone_gps
            if self._DroneGPSinIn[num].isNew():
                self._d_DroneGPSin[num] = self._DroneGPSinIn[num].read()
                self._drone_gps[num] = True
                #logger.debug("Drone%dGPS:  % f % f % f",
                #             (num + 1),
                #             self._d_DroneGPSin[num].latitude,
                #             self._d_DroneGPSin[num].longitude,
                #             self._d_DroneGPSin[num].altitude)

            # read_target_gps
            if self._TargetGPSinIn[num].isNew():
                self._d_TargetGPSin[num] = self._TargetGPSinIn[num].read()
                self._target_gps[num] = True
                #logger.debug("Target%dGPS:   % f % f % f",
                #             (num + 1),
                #             self._d_TargetGPSin[num].latitude,
                #             self._d_TargetGPSin[num].longitude,
                #             self._d_TargetGPSin[num].altitude)

            # read_mag
            if self._inMagIn[num].isNew():
                self._d_inMag[num] = self._inMagIn[num].read()
                self._mag[num] = True
                #logger.debug("Mag%d: %d",
                #             (num + 1),
                #             self._d_inMag[num].data)

            # read_state
            if self._inStateIn[num].isNew():
                self._d_inState[num] = self._inStateIn[num].read()
                self._state[num] = True
                #logger.debug("State%d: %s",
                #             (num + 1),
                #             self._d_inState[num].data)

            num += 1
 
    def _set_map_data(self):
        self.map_zoom_update.map_zoom(self._MapZoomLevel[0])

        num = 0
        while num < self._Drone_Num:
            if self._drone_gps[num]:
                self.drone_pos_update.drone_move(self._d_DroneGPSin[num].latitude, self._d_DroneGPSin[
                    num].longitude, self._d_DroneGPSin[num].altitude, num)
            if self._target_gps[num]:
                self.target_pos_update.drone_target(
                    self._d_TargetGPSin[num].latitude, self._d_TargetGPSin[num].longitude, num)
            if self._mag[num]:
                self.drone_angle_update.drone_rotate(
                    self._d_inMag[num].data, num)
            if self._state[num]:              
                self.drone_state_update.drone_state(self._d_inState[num].data, num)
            num += 1

def DroneMapViewerInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=dronemapviewer_spec)
    manager.registerFactory(profile,
                            DroneMapViewer,
                            OpenRTM_aist.Delete)

def MyModuleInit(manager):
    DroneMapViewerInit(manager)

    # Create a component
    comp = manager.createComponent("DroneMapViewer")

def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager(True)
    map_create.MapCreateApp().run()

if __name__ == "__main__":
    main()

