#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

import unittest

import RTC
import OpenRTM_aist
import DroneMapViewer

import sys

from mock import Mock
from mock import patch, MagicMock

INPORT_SET_NUM = 3

class TestDroneMapViewer(unittest.TestCase):
    def setUp(self):
        self.m = DroneMapViewer.DroneMapViewer(0)

        self.xx_logger = Mock()
        DroneMapViewer.logger.warning = self.xx_logger.warning
        DroneMapViewer.logger.error = self.xx_logger.error
        DroneMapViewer.logger.info = self.xx_logger.info
        DroneMapViewer.logger.debug = self.xx_logger.debug

    def test_read_inport(self):

        m_drone = [0] * INPORT_SET_NUM
        m_target = [0] * INPORT_SET_NUM
        m_mag = [0] * INPORT_SET_NUM
        m_state = [0] * INPORT_SET_NUM

        num = 0
        while num < INPORT_SET_NUM:
            m_drone[num] = Mock()
            m_target[num] = Mock()
            m_mag[num] = Mock()
            m_state[num] = Mock()
            self.m._DroneGPSinIn[num].isNew = m_drone[num].isNew
            self.m._TargetGPSinIn[num].isNew = m_target[num].isNew
            self.m._inMagIn[num].isNew = m_mag[num].isNew
            self.m._inStateIn[num].isNew = m_state[num].isNew
            num += 1

        # inport:
        drone_is_new = [0] * INPORT_SET_NUM
        drone_is_new[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        drone_is_new[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        drone_is_new[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        target_is_new = [0] * INPORT_SET_NUM
        target_is_new[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        target_is_new[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        target_is_new[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        mag_is_new = [0] * INPORT_SET_NUM
        mag_is_new[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        mag_is_new[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        mag_is_new[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        state_is_new = [0] * INPORT_SET_NUM
        state_is_new[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        state_is_new[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        state_is_new[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]


        # result:
        drone_result = [0] * INPORT_SET_NUM
        drone_result[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        drone_result[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        drone_result[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        target_result = [0] * INPORT_SET_NUM
        target_result[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        target_result[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        target_result[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        mag_result = [0] * INPORT_SET_NUM
        mag_result[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        mag_result[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        mag_result[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]

        state_result = [0] * INPORT_SET_NUM
        state_result[0] = [False, True, False, False, True, True, False, False, False, False, True, True, True, True]
        state_result[1] = [None, None, False, True, False, True, False, False, True, True, False, False, True, True]
        state_result[2] = [None, None, None, None, None, None, False, True, False, True, False, True, False, True]


        #
        count = 1
        elem = 0
        drone_num = 1
        while drone_num < 4:
            # conf:
            self.m._Drone_Num = drone_num
            count = (2 * count)
            i = 0
            while i < count:

                num = 0
                while num < self.m._Drone_Num:
                    m_drone[num].isNew.return_value = drone_is_new[num][elem]
                    m_target[num].isNew.return_value = target_is_new[num][elem]
                    m_mag[num].isNew.return_value = mag_is_new[num][elem]
                    m_state[num].isNew.return_value = state_is_new[num][elem]
                    num += 1

                self.m._read_inport()

                num = 0
                while num < self.m._Drone_Num:
                    self.assertEqual(self.m._drone_gps[num], drone_result[num][elem])
                    self.assertEqual(self.m._target_gps[num], target_result[num][elem])
                    self.assertEqual(self.m._mag[num], mag_result[num][elem])
                    self.assertEqual(self.m._state[num], state_result[num][elem])
                    num += 1

                i += 1
                elem += 1
            drone_num += 1

if __name__ == '__main__':
    unittest.main()
