import kivy
kivy.require('1.0.8')

from kivy.app import App

from kivy.uix.label import Label
from kivy.uix.widget import Widget

from kivy.uix.gridlayout import GridLayout

from kivy.garden.mapview import MapView, MapMarker
from kivy.graphics import Color, Line, Rectangle, Rotate, PushMatrix, PopMatrix

from kivy.core.window import Window
from kivy.clock import Clock


import time
import math


MAP_SIZE_WIDTH = 880
MAP_SIZE_HEIGHT = 660

READ_CHECK_TIME = 20.0


class DroneNumSet():

    def drone_num(self, num):
        global drone_num
        drone_num = num
        print "drone_num:",drone_num


class DronePosUpdate():

    def drone_move(self, lat, lon, alt, no):
        drone_lat[no] = lat
        drone_lon[no] = lon
        drone_alt[no] = alt


class DroneAngleUpdate():

    def drone_rotate(self, angle, no):
        drone_angle[no] = angle


class TargetPosUpdate():

    def drone_target(self, target_lat, target_lon, no):
        drone_target_lat[no] = target_lat
        drone_target_lon[no] = target_lon


class DroneStateUpdate():

    def drone_state(self, state, no):
        drone_state[no] = state


class MapZoomUpdate():

    def map_zoom(self, zoom):
        global map_zoom_level
        map_zoom_level = zoom


class MapDisplay(Widget):

    def __init__(self, **kwargs):
        super(MapDisplay, self).__init__(**kwargs)

        self._calc_c_pos = [False] * drone_num

        self.map_drone_width = [0] * drone_num
        self.map_drone_height = [0] * drone_num
        self.map_target_width = [0] * drone_num
        self.map_target_height = [0] * drone_num

        self.angle = [0] * drone_num
        self.mapmarker = [0] * drone_num

        self.select_drone_calc_c_pos()
        self.map_center_calc()

        self.mapview = MapView(zoom=map_zoom_level, lat=drone_lat[0], lon=drone_lon[0])
        self.mapview.size = (MAP_SIZE_WIDTH, MAP_SIZE_HEIGHT)
        self.mapview.center_on(self.map_center_lat, self.map_center_lon)

        self.target_update()

        num = 0
        while num < drone_num:
            self.angle[num] = drone_angle[num]

            self.mapmarker[num] = MapMarker(anchor_x=0.5, anchor_y=0.5, lat=drone_lat[num], lon=drone_lon[num],
                                            source="icon/drone%d.png" % (num + 1))

            with self.mapmarker[num].canvas.before:
                PushMatrix()
                Rotate(angle=self.angle[num], axis=(
                    0, 0, -1), origin=(self.map_drone_width[num], self.map_drone_height[num]))

            with self.mapmarker[num].canvas.after:
                PopMatrix()
                Color(rgba=(0.3, 0.6, 0.2, 0.9))
                Line(points=[self.map_drone_width[num], self.map_drone_height[num],
                             self.map_target_width[num], self.map_target_height[num]], width=1.5)

            self.mapview.add_marker(self.mapmarker[num])
            num += 1

        self.add_widget(self.mapview)

    def _is_all_rtl(self):
        all_rtl = True
        num = 0
        while num < drone_num:
            if drone_state[num] != "RTL" and drone_state[num] != "Disarm":
                all_rtl = False          
            num += 1
        return all_rtl

    def select_drone_calc_c_pos(self):
        all_rtl = self._is_all_rtl()
        if all_rtl == False:
            num = 0
            while num < drone_num:
                if drone_state[num] != "RTL" and drone_state[num] != "Disarm":
                    self._calc_c_pos[num] = True
                else:
                    self._calc_c_pos[num] = False            
                num += 1

    def map_update(self):
        self.target_update()
        self.icon_update()

        if self.map_center_change():
            print "===========MAP_UPDATE=============="
            self.map_center_calc()

        self.mapview.zoom = map_zoom_level
        self.mapview.center_on(self.map_center_lat, self.map_center_lon)

    def map_center_change(self):
        self.select_drone_calc_c_pos()
        ret = False
        num = 0
        while num < drone_num:
            if self._calc_c_pos[num]:
                if (self.map_drone_height[num] < 100 or self.map_drone_height[num] > (MAP_SIZE_HEIGHT - 100) or
                        self.map_drone_width[num] < 100 or self.map_drone_width[num] > (MAP_SIZE_WIDTH - 100)):
                    ret = True
                    break
            num += 1

        return ret

    def map_center_calc(self):
        lat = 0
        lon = 0
        num = 0
        control_num = 0
        while num < drone_num:
            if self._calc_c_pos[num]:
                lat += drone_lat[num]
                lon += drone_lon[num]
                control_num += 1
            num += 1

        self.map_center_lat = lat/control_num
        self.map_center_lon = lon/control_num

    def icon_update(self):
        num = 0
        while num < drone_num:

            self.mapmarker[num].lat = drone_lat[num]
            self.mapmarker[num].lon = drone_lon[num]
            self.angle[num] = drone_angle[num]

            self.mapmarker[num].canvas.before.clear()
            self.mapmarker[num].canvas.after.clear()

            with self.mapmarker[num].canvas.before:
                PushMatrix()
                Rotate(angle=self.angle[num], axis=(
                    0, 0, -1), origin=(self.map_drone_width[num], self.map_drone_height[num]))

            with self.mapmarker[num].canvas.after:
                PopMatrix()
                Color(rgba=(0.3, 0.6, 0.2, 0.9))
                Line(points=[self.map_drone_width[num], self.map_drone_height[num],
                             self.map_target_width[num], self.map_target_height[num]], width=1.5)

            num += 1

    def target_update(self):
        min_lon = (180 / math.pow(2, map_zoom_level + 7)) * \
            ((MAP_SIZE_WIDTH / 2) * -1) + self.map_center_lon
        min_lat = (180 / math.pi) * math.asin(math.tanh((math.pi / math.pow(2, map_zoom_level + 7)) * ((MAP_SIZE_HEIGHT / 2) * -1) +
                                                        math.atanh(math.sin((math.pi / 180) * self.map_center_lat))))
        max_lon = (180 / math.pow(2, map_zoom_level + 7)) * \
            ((MAP_SIZE_WIDTH / 2)) + self.map_center_lon
        max_lat = (180 / math.pi) * math.asin(math.tanh((math.pi / math.pow(2, map_zoom_level + 7)) * ((MAP_SIZE_HEIGHT / 2)) +
                                                        math.atanh(math.sin((math.pi / 180) * self.map_center_lat))))

        num = 0
        while num < drone_num:
            self.map_drone_width[num] = (MAP_SIZE_WIDTH / 2 + (MAP_SIZE_WIDTH * (drone_lon[num] - self.map_center_lon)) /
                                         math.fabs(max_lon - min_lon))
            self.map_drone_height[num] = (MAP_SIZE_HEIGHT / 2 + (MAP_SIZE_HEIGHT * (drone_lat[num] - self.map_center_lat)) /
                                          math.fabs(max_lat - min_lat))

            self.map_target_width[num] = (MAP_SIZE_WIDTH / 2 + (MAP_SIZE_WIDTH * (drone_target_lon[num] - self.map_center_lon)) /
                                          math.fabs(max_lon - min_lon))
            self.map_target_height[num] = (MAP_SIZE_HEIGHT / 2 + (MAP_SIZE_HEIGHT * (drone_target_lat[num] - self.map_center_lat)) /
                                           math.fabs(max_lat - min_lat))

            num += 1


class DroneInfo(GridLayout):

    def __init__(self, **kwargs):
        super(DroneInfo, self).__init__(**kwargs)

        self.info_pos_x = [0] * drone_num
        self.info_pos_y = [0] * drone_num

        self.drone_info = [0] * drone_num
        self.info_name = [0] * drone_num
        self.info_lat = [0] * drone_num
        self.info_lon = [0] * drone_num
        self.info_alt = [0] * drone_num

        num = 0
        while num < drone_num:
            self.info_pos_x[num] = MAP_SIZE_WIDTH - 145
            self.info_pos_y[num] = MAP_SIZE_HEIGHT - (115 * (num + 1))

            self.drone_info[num] = Label()
            with self.drone_info[num].canvas.before:
                Color(rgba=(0, 0, 0, 0.5))
                Rectangle(size=(140, 110), pos=(
                    self.info_pos_x[num], self.info_pos_y[num]))

            self.info_name[num] = Label(text="Drone%d" % (num + 1), boil=True, font_size=20, text_size=(180, None),
                                   pos=(self.info_pos_x[num] + 50, self.info_pos_y[num] + 47))
            self.info_lat[num] = Label(text="lat:  %f" % drone_lat[num], font_size=15, text_size=(180, None),
                                       pos=(self.info_pos_x[num] + 60, self.info_pos_y[num] + 17))
            self.info_lon[num] = Label(text="lon: %f" % drone_lon[num], font_size=15, text_size=(180, None),
                                       pos=(self.info_pos_x[num] + 60, self.info_pos_y[num] - 8))
            self.info_alt[num] = Label(text="alt:  %f" % drone_alt[num], font_size=15, text_size=(180, None),
                                       pos=(self.info_pos_x[num] + 60, self.info_pos_y[num] - 33))

            self.drone_info[num].add_widget(self.info_name[num])
            self.drone_info[num].add_widget(self.info_lat[num])
            self.drone_info[num].add_widget(self.info_lon[num])
            self.drone_info[num].add_widget(self.info_alt[num])

            self.add_widget(self.drone_info[num])
            num += 1

    def info_update(self):
        num = 0
        while num < drone_num:
            self.info_lat[num].text = "lat:  %f" % drone_lat[num]
            self.info_lon[num].text = "lon: %f" % drone_lon[num]
            self.info_alt[num].text = "alt:  %f" % drone_alt[num]

            num += 1


class MapLoadTimeout(Widget):

    def __init__(self, **kwargs):
        super(MapLoadTimeout, self).__init__(**kwargs)

        self.timeout_msg = Label(text="Map Loaded Timeout", font_size=self.height / 2,
                                 halign="center", pos=(MAP_SIZE_WIDTH / 2.2, MAP_SIZE_HEIGHT / 2.2))

        self.add_widget(self.timeout_msg)


class MapCreateApp(App):

    def __init__(self, **kwargs):
        super(MapCreateApp, self).__init__(**kwargs)
        
        global drone_num
        drone_num = 0
        while True:
            if drone_num > 0:
                break

        global drone_lat, drone_lon, drone_alt, drone_target_lat, drone_target_lon, drone_angle, drone_state
        drone_lat = [0] * drone_num
        drone_lon = [0] * drone_num
        drone_alt = [0] * drone_num
        drone_target_lat = [0] * drone_num
        drone_target_lon = [0] * drone_num
        drone_angle = [0] * drone_num
        drone_state = ["None"] * drone_num

        check_start = time.time()
        self.timeout = False
        num = 0
        while num < drone_num:
            if (drone_lat[num] and drone_lon[num] and drone_alt[num] and drone_angle[num] and
                    drone_target_lat[num] and drone_target_lon[num]):
                num += 1

            if (time.time() - check_start) > READ_CHECK_TIME:
                self.timeout = True
                break

    def build(self):
        self.title = "DroneMapViewer"
        Window.size = MAP_SIZE_WIDTH, MAP_SIZE_HEIGHT

        if self.timeout:
            root = MapLoadTimeout()
        else:
            self.map = MapDisplay()
            self.info = DroneInfo()
            root = self.map
            root.add_widget(self.info)

            Clock.schedule_interval(self.data_update, 0.1)

        return root

    def data_update(self, dt):
        self.map.map_update()
        self.info.info_update()


if __name__ == '__main__':
    MapCreateApp().run()
