// -*- Java -*-
/*!
 * @file  DroneMathmetics.java
 * @brief ModuleDescription
 * @date  $Date$
 *
 * $Id$
 */

/*!
 * @class DroneMathmetics
 * @brief ModuleDescription
 *
 */

public class DroneMathmetics {

	public Double distance_math(Double hm_lon, Double dr_lon, Double hm_lat,
			Double dr_lat, Double hm_alt, Double dr_alt) {

		if (gpsdata_check(hm_lon, dr_lon, hm_lat, dr_lat) == false) {
			return 0.0;
		} else {
			Double distance = distance_calculate(hm_lon, dr_lon, hm_lat,
					dr_lat, hm_alt, dr_alt);
			return distance;
		}

	}

	private Double distance_calculate(Double hm_lon, Double dr_lon,
			Double hm_lat, Double dr_lat, Double hm_alt, Double dr_alt) {
		// Equatorial radius [m]
		Double a = 6378137.000;
		// Polar radius [m]
		Double b = 6356752.314;
		// major eccentricity
		Double e = Math
				.sqrt((Math.pow(a, 2) - Math.pow(b, 2)) / Math.pow(a, 2));

		Double x1 = hm_lon;
		Double y1 = hm_lat;

		Double x2 = dr_lon;
		Double y2 = dr_lat;

		Double dx = null;

		if (Math.abs(x1 - x2) > 180.000) {
			dx = Math.toRadians((360.000 - Math.abs(x1 - x2)));
		} else {
			dx = Math.toRadians(x1 - x2);
		}

		Double dy = Math.toRadians(y1 - y2);
		Double dz = Math.abs(hm_alt - dr_alt);

		Double uy = Math.toRadians((y1 + y2) / 2.0);

		Double W = Math
				.sqrt(1 - (Math.pow(e, 2) * (Math.pow(Math.sin(uy), 2))));

		// Meridian radius of curvature
		Double M = (a * (1 - Math.pow(e, 2))) / Math.pow(W, 3);

		// Prime vertical radius of curvature
		Double N = a / W;

		Double d = Math.sqrt(Math.pow((dy * M), 2)
				+ Math.pow((dx * N * Math.cos(uy)), 2));

		Double distance = Math.sqrt(Math.pow(d, 2) + Math.pow(dz, 2));

		return distance;
	}

	private Boolean gpsdata_check(Double hm_lon, Double dr_lon, Double hm_lat,
			Double dr_lat) {

		if (-180.000 <= hm_lon && hm_lon <= 180.000) {

		} else {
			System.out.println("home longitude input error");
			return false;
		}

		if (-180.000 <= dr_lon && dr_lon <= 180.000) {

		} else {
			System.out.println("drone longitude input error");
			return false;
		}

		if (-90.000 <= hm_lat && hm_lat <= 90.000) {

		} else {
			System.out.println("home latitude input error");
			return false;
		}

		if (-90.000 <= dr_lat && dr_lat <= 90.000) {

		} else {
			System.out.println("drone latitude input error");
			return false;
		}

		return true;
	}

	public static void main() {
		System.out.println("f = DroneMathmetics()");
		DroneMathmetics f = new DroneMathmetics();
		Double distance = f.distance_math(139.929727, 139.767374, 37.494846,
				35.681043, 0.0, 0.0);
		System.out.println("distance:" + (distance / 1000) + "[km]");
	}
}