// -*- C++ -*-
/*!
 * @file  RTC_GameController_Win.cpp
 * @brief RTC_GameController Windows
 * @date $Date$
 *
 * @author AIZUK
 * kohno@aizuk.jp
 *
 * $Id$
 */

#include "RTC_GameController_Win.h"
#pragma comment(lib, "winmm.lib")
#include <mmsystem.h>

#include <windows.h>
#define AN_MAX 32767
#define AN_MIN -32767
#define AN_X_ZERO 0.2
#define AN_Y_ZERO 0.2
JOYINFOEX js0;
double an_LX = 0;
double an_LY = 0;
double an_RX = 0;
double an_RY = 0;
double an_L2 = 0;
double an_R2 = 0;
double POV[2] = { 0, 0 };

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_gamecontroller_win_spec[] =
  {
    "implementation_id", "RTC_GameController_Win",
    "type_name",         "RTC_GameController_Win",
    "description",       "RTC_GameController Windows",
    "version",           "1.0.0",
    "vendor",            "AIZUK",
    "category",          "Controller",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Stick_Gain", "1.0",
    // Widget
    "conf.__widget__.Stick_Gain", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_GameController_Win::RTC_GameController_Win(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Controller_TypeOut("Controller_Type", m_Controller_Type),
    m_ButtonOut("Button", m_Button),
    m_AnalogOut("Analog", m_Analog)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_GameController_Win::~RTC_GameController_Win()
{
}



RTC::ReturnCode_t RTC_GameController_Win::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("Controller_Type", m_Controller_TypeOut);
  addOutPort("Button", m_ButtonOut);
  addOutPort("Analog", m_AnalogOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  m_Analog.data.length(11);
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Stick_Gain", m_Stick_Gain, "1.0");
  // </rtc-template>
  
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onFinalize()
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onExecute(RTC::UniqueId ec_id)
{
	js0.dwSize = sizeof(JOYINFOEX);
	js0.dwFlags = JOY_RETURNALL;
	joyGetPosEx(0, &js0);
	double pre[10];
	an_LX = ((double)(js0.dwXpos) - 32767);
	if (an_LX>AN_MAX)an_LX = AN_MAX;
	if (an_LX<AN_MIN)an_LX = AN_MIN;
	an_LX = an_LX / 32767;
	if (an_LX<AN_X_ZERO&&an_LX>-AN_X_ZERO)an_LX = 0;

	an_LY = ((double)(js0.dwYpos) - 32767);
	if (an_LY>AN_MAX)an_LY = AN_MAX;
	if (an_LY<AN_MIN)an_LY = AN_MIN;
	an_LY = an_LY / 32767;
	if (an_LY<AN_Y_ZERO&&an_LY>-AN_Y_ZERO)an_LY = 0;
	an_LX *= m_Stick_Gain;
	an_LY *= m_Stick_Gain;
	printf("js0.ANLX:%f:\n\r",an_LX);
	printf("js0.ANLY:%f:\n\r",an_LY);
	printf("\n\n\n\r");


	an_RX = ((double)(js0.dwZpos) - 32767);
	if (an_RX>AN_MAX)an_RX = AN_MAX;
	if (an_RX<AN_MIN)an_RX = AN_MIN;
	an_RX = an_RX / 32767;
	if (an_RX<AN_X_ZERO&&an_RX>-AN_X_ZERO)an_RX = 0;
	an_RY = ((double)(js0.dwRpos) - 32767);
	if (an_RY>AN_MAX)an_RY = AN_MAX;
	if (an_RY<AN_MIN)an_RY = AN_MIN;
	an_RY = an_RY / 32767;
	if (an_RY<AN_Y_ZERO&&an_RY>-AN_Y_ZERO)an_RY = 0;
	an_RX *= m_Stick_Gain;
	an_RY *= m_Stick_Gain;
	
	printf("js0.ANRX:%f:%f\n\r",an_RX);
	printf("js0.ANRY:%f:%f\n\r",an_RY);
	printf("\n\n\n\r");
	
	an_L2 = ((double)(js0.dwUpos) - 32767);
	if (an_L2>AN_MAX)an_L2 = AN_MAX;
	if (an_L2<AN_MIN)an_L2 = AN_MIN;
	an_L2 = an_L2 / 32767;
	if (an_L2<AN_X_ZERO&&an_L2>-AN_X_ZERO)an_L2 = 0;

	an_R2 = ((double)(js0.dwVpos) - 32767);
	if (an_R2>AN_MAX)an_R2 = AN_MAX;
	if (an_R2<AN_MIN)an_R2 = AN_MIN;
	an_R2 = an_R2 / 32767;
	if (an_R2<AN_Y_ZERO&&an_R2>-AN_Y_ZERO)an_R2 = 0;
	
	printf("js0.ANL2:%f:%f\n\r",an_L2);
	printf("js0.ANR2:%f:%f\n\r",an_R2);
	printf("\n\n\n\r");
	

	printf("POV:%x\n\n\n\r",js0.dwPOV);
	if (js0.dwPOV == 65535){
		POV[0] = 0;
		POV[1] = 0;
	}
	else if (js0.dwPOV == 0){
		POV[0] = -1;
		POV[1] = 0;
	}
	else if (js0.dwPOV == 4500){
		POV[0] = -1;
		POV[1] = 1;
	}
	else if (js0.dwPOV == 9000){
		POV[0] = 0;
		POV[1] = 1;
	}
	else if (js0.dwPOV == 13500){
		POV[0] = 1;
		POV[1] = 1;
	}
	else if (js0.dwPOV == 18000){
		POV[0] = 1;
		POV[1] = 0;
	}
	else if (js0.dwPOV == 22500){
		POV[0] = 1;
		POV[1] = -1;
	}
	else if (js0.dwPOV == 27000){
		POV[0] = 0;
		POV[1] = -1;
	}
	else if (js0.dwPOV == 31500){
		POV[0] = -1;
		POV[1] = -1;
	}
//	printf("================%d=======%d============-\n\r", js0.dwPOV, POV[1]);
	m_Analog.data[0] = POV[0];
	m_Analog.data[1] = POV[1];
	m_Analog.data[2] = an_LY;
	m_Analog.data[3] = an_LX;
	m_Analog.data[4] = an_RY;
	m_Analog.data[5] = an_RX;
	m_Analog.data[6] = an_L2;
	m_Analog.data[7] = an_R2;
	m_AnalogOut.write();
	m_Button.data = js0.dwButtons;
	printf("Button:%x\n\r", m_Button.data);
	m_ButtonOut.write();
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTC_GameController_Win::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}



extern "C"
{
 
  void RTC_GameController_WinInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_gamecontroller_win_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_GameController_Win>,
                             RTC::Delete<RTC_GameController_Win>);
  }
  
};


